/*
 * 18:12:25 12/06/99
 *
 * Ship.java - A standard ship
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.chez.powerteam.retaliator;

import java.awt.Image;
import java.awt.Point;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.image.ImageObserver;

/**
 * A single class which can be used for Player, Ennemies, Laser...
 * This class can be used, in fact, for any kind of animated 2D
 * object.
 * @author Romain Guy
 * @version 1.5.1
 * @see Laser Player
 */

public class Ship
{
  private Image img;
  private int positionX;
  private int positionY;

  /**
   * This constant defines a clipped Ship
   */

  public static final int NO_CLIP = 0;

  /**
   * This constant defines a Ship bounded by the drawing area
   */

  public static final int BOUNDARY_SCREEN = 1;


  private int maxX, maxY, shipWidth, shipHeight;

  /**
   * A simple constructor which needs some informations about
   * the painting context.
   * @param img The Image used to represent one Ship
   * @param width The width of the area where the Ship will be painted
   * @param height The height of the area where the Ship will be painted
   * @param o The ImageObserver used to paint the Ship
   */

  public Ship(Image img, int width, int height, ImageObserver o)
  {
    this.img = img;
    maxX = width;
    maxY = height;
    shipWidth = img.getWidth(o);
    shipHeight = img.getHeight(o);
  }

  /**
   * Move the Ship on the drawing are while increasing its x and y positions.
   * The mode determines if the Ship is bounded in the painting area or if it can
   * get out of it.
   * @param incrementX The value added to the current X position
   * @param incrementY The value added to the current Y position
   * @param mode The drawing mode: <code>NO_CLIP</code> or <code>BOUNDARY_SCREEN</code>
   */

  public void move(int incrementX, int incrementY, int mode)
  {
    positionX += incrementX;
    positionY += incrementY;
    testPosition(mode);
  }

  /**
   * Free the memory used by the Ship. Do not forget to call it
   * when your terminate your application !
   */

  public void free()
  {
    img.flush();
    img = null;
  }

  /**
   * Return the Ship's current X position.
   */

  public int getPositionX()
  {
    return positionX;
  }

  /**
   * Return the Ship's current Y position.
   */

  public int getPositionY()
  {
    return positionY;
  }

  /**
   * Return the Ship's current position.
   */

  public Point getPosition()
  {
    return new Point(positionX, positionY);
  }

  /**
   * Set Ship's X position.
   * @param x New X position
   * @param mode The drawing mode: <code>NO_CLIP</code> or <code>BOUNDARY_SCREEN</code>
   */

  public void setPositionX(int x, int mode)
  {
    positionX = x;
    testPosition(mode);
  }

  /**
   * Set Ship's Y position.
   * @param y New Y position
   * @param mode The drawing mode: <code>NO_CLIP</code> or <code>BOUNDARY_SCREEN</code>
   */

  public void setPositionY(int y, int mode)
  {
    positionY = y;
    testPosition(mode);
  }

  /**
   * Set Ship's position.
   * @param x New X position
   * @param y New Y position
   * @param mode The drawing mode: <code>NO_CLIP</code> or <code>BOUNDARY_SCREEN</code>
   */

  public void setPosition(int x, int y, int mode)
  {
    positionX = x;
    positionY = y;
    testPosition(mode);
  }

  /**
   * Set Ship's position.
   * @param xy A Point defining the new position
   * @param mode The drawing mode: <code>NO_CLIP</code> or <code>BOUNDARY_SCREEN</code>
   */

  public void setPosition(Point xy, int mode)
  {
    setPosition(xy.x, xy.y, mode);
  }

  /**
   * If <code>mode</code> is different from <code>BOUNDARY_SCREEN</code> the Ship
   * can be clipped by the drawing area's borders. Otherwise, Ship is placed in
   * the area, close to the nearest border.
   * @param mode The drawing mode: <code>NO_CLIP</code> or <code>BOUNDARY_SCREEN</code>
   */

  public void testPosition(int mode)
  {
    if (mode == BOUNDARY_SCREEN)
    {
      if (positionX > maxX - shipWidth) positionX = maxX - shipWidth;
      if (positionX < 0) positionX = 0;
      if (positionY > maxY) positionY = maxY - shipHeight;
      if (positionY < 0) positionY = 0;
    }
  }

  /**
   * Change the Ship's picture. Used to destroy a Ship (and so display an
   * explosion...)
   * @param img The new Image
   */

  public void setImage(Image img)
  {
    this.img = img;
  }

  /**
   * Paints the Ship.
   * @param g The Graphics where to paint the Ship
   * @param o The ImageObserver used to render the Ship's picture
   */

  public void paint(Graphics g, ImageObserver o)
  {
    g.drawImage(img, positionX, positionY, o);
  }
}

// End of Ship.java
