/*
 * 18:07:21 12/06/99
 *
 * StarField.java - A layered Starfield !!
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.chez.powerteam.retaliator;

import java.awt.Color;
import java.awt.Graphics;

/**
 * StarField is a very useful class which can create and display a
 * three layered starfield.
 * @author Romain Guy
 * @version 2.1
 */

public class StarField
{
  private boolean drawStatus;

  private int[][] starFieldLayer = new int[3][];
  private int[][] starFieldLayerOrigin = new int[3][];

  private int nbStars;
  private int starMove[] = new int[3];
  private int starColor[] = new int[3];

  private int height, width, surface;

  /**
   * Defines a vertically scrolling starfield.
   */

  public static final int VERTICAL = 1;

  /**
   * Defines a horizontally scrolling starfield.
   */

  public static final int HORIZONTAL = 2;

  /**
   * A StarField is made of three different layers of stars. Do not forget that
   * the two first arguments are arrays of <code>int</code>. A color is
   * constructed like this: <code>new Color(color, color, color)</code>.
   * This means you can only used gray colors. But you can fade'em while
   * using the <code>setColors(int[])</code> method.
   * @param color An array of 3 int corresponding to the color of each layer
   * @param move An array of 3 int corresponding to the move step of each layer
   * @param nb The amount of stars (same on each layer)
   * @param dir The direction of the stars. Use: <code>VERTICAL</code> or <code>HORIZONTAL</code>
   * @param w The width of the StarField
   * @param h The height of the StarField
   */

  public StarField(int[] color, int[] move, int nb, int dir, int w, int h)
  {
    nbStars = nb;
    width = w;
    height = h;
    surface = w * h;

    for (int i = 0; i < 3; i++)
    {
      starMove[i] = move[i];
      starColor[i] = color[i];
      starFieldLayer[i] = new int[nbStars];
      starFieldLayerOrigin[i] = new int[nbStars];
    }

    if (dir == VERTICAL)
      for (int i = 0; i < 3; i++)
        starMove[i] *= width;

    initStarField();
  }

  /**
   * Initialize a new StarField. In fact, it only randomize stars
   * positions on each layer. Call it whenever you want...
   */

  public void initStarField()
  {
    for (int i = 0; i < nbStars; i++)
    {
      for (int j = 0; j < 3; j++)
      {
        starFieldLayer[j][i] = (int) (Math.random() * surface);
        starFieldLayerOrigin[j][i] = (int) (Math.random() * width);
      }
    }
  }

  /**
   * Display the StarField.
   * @param g The Graphics context where to paint the starfield
   */

  public void paint(Graphics g)
  {
    drawStatus = true;
    for (int i = 0; i < nbStars; i++)
    {
      for (int j = 0; j < 3; j++)
      {
        if (starFieldLayer[j][i] < surface - starMove[j])
          starFieldLayer[j][i] += starMove[j];
        else
          starFieldLayer[j][i] = starFieldLayerOrigin[j][i];

        g.setColor(new Color(starColor[j], starColor[j], starColor[j]));
	int x = starFieldLayer[j][i] % width;
	int y = starFieldLayer[j][i] / width;
        g.drawLine(x, y, x, y);
      }
    }
    drawStatus = false;
  }

  /**
   * Returns <code>true</code> if the StardField is currently being rendered.
   * You should call the <code>paint(Graphics)</code> methods only when this
   * method returns <code>false</code>. Otherwise, you could see lines on the
   * screen...
   */

  public boolean isDrawing()
  {
    return drawStatus;
  }

  /**
   * Call it to change the colors of the stars. Very useful to fade the
   * StarField.
   * color An array of 3 int corresponding to the color of each layer of stars
   */

  public void setColors(int[] color)
  {
    for (int i = 0; i < 3; i++)
      starColor[i] = color[i];
  }

  /**
   * Return an array of 3 int containing the color of each layer.
   */

  public int[] getColors()
  {
    return starColor;
  }

  /**
   * With this method, you can change the 'speed' of the stars. For example,
   * at the end of a level, increase moves to make the starship flying at an
   * incredible speed !
   * @param move An array of 3 int corresponding to the move step of each layer of stars
   */

  public void setMoves(int[] move)
  {
    for (int i = 0; i < 3; i++)
      starMove[i] = move[i];
  }

  /**
   * Return an array of 3 int containing the move step of each layer.
   */

  public int[] getMoves()
  {
    return starMove;
  }

  /**
   * Free the memory used by the StarField.
   */

  public void free()
  {
    starMove = null;
    starColor = null;
    starFieldLayer = null;
    starFieldLayerOrigin = null;
  }
}

// End of StarField.java
